/****************************************************************************
** Meta object code from reading C++ file 'searchworker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../util-dfm/src/dfm-search/dfm-search-lib/core/searchstrategy/searchworker.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto dfmsearch::SearchWorker::qt_create_metaobjectdata<qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dfmsearch::SearchWorker",
        "resultFound",
        "",
        "dfmsearch::SearchResult",
        "result",
        "searchFinished",
        "dfmsearch::SearchResultList",
        "results",
        "errorOccurred",
        "dfmsearch::SearchError",
        "error",
        "doSearch",
        "dfmsearch::SearchQuery",
        "query",
        "dfmsearch::SearchOptions",
        "options",
        "dfmsearch::SearchType",
        "searchType",
        "cancelSearch"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resultFound'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchResult &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchResultList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchError &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'doSearch'
        QtMocHelpers::SlotData<void(const dfmsearch::SearchQuery &, const dfmsearch::SearchOptions &, dfmsearch::SearchType)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'cancelSearch'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SearchWorker, qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dfmsearch::SearchWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>.metaTypes,
    nullptr
} };

void dfmsearch::SearchWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchWorker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resultFound((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchResult>>(_a[1]))); break;
        case 1: _t->searchFinished((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchResultList>>(_a[1]))); break;
        case 2: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchError>>(_a[1]))); break;
        case 3: _t->doSearch((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchQuery>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchOptions>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchType>>(_a[3]))); break;
        case 4: _t->cancelSearch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< dfmsearch::SearchResult >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< dfmsearch::SearchError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SearchWorker::*)(const dfmsearch::SearchResult & )>(_a, &SearchWorker::resultFound, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchWorker::*)(const dfmsearch::SearchResultList & )>(_a, &SearchWorker::searchFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchWorker::*)(const dfmsearch::SearchError & )>(_a, &SearchWorker::errorOccurred, 2))
            return;
    }
}

const QMetaObject *dfmsearch::SearchWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmsearch::SearchWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchWorkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmsearch::SearchWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmsearch::SearchWorker::resultFound(const dfmsearch::SearchResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void dfmsearch::SearchWorker::searchFinished(const dfmsearch::SearchResultList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void dfmsearch::SearchWorker::errorOccurred(const dfmsearch::SearchError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
