/****************************************************************************
** Meta object code from reading C++ file 'dsearch_global.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../util-dfm/include/dfm-search/dfm-search/dsearch_global.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dsearch_global.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9dfmsearchE_t {};
} // unnamed namespace

template <> constexpr inline auto dfmsearch::qt_create_metaobjectdata<qt_meta_tag_ZN9dfmsearchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dfmsearch",
        "SearchType",
        "FileName",
        "Content",
        "Custom",
        "SearchStatus",
        "Ready",
        "Searching",
        "Finished",
        "Cancelled",
        "Error",
        "SearchMethod",
        "Indexed",
        "Realtime"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SearchType'
        QtMocHelpers::EnumData<SearchType>(1, 1, QMC::EnumFlags{}).add({
            {    2, SearchType::FileName },
            {    3, SearchType::Content },
            {    4, SearchType::Custom },
        }),
        // enum 'SearchStatus'
        QtMocHelpers::EnumData<SearchStatus>(5, 5, QMC::EnumFlags{}).add({
            {    6, SearchStatus::Ready },
            {    7, SearchStatus::Searching },
            {    8, SearchStatus::Finished },
            {    9, SearchStatus::Cancelled },
            {   10, SearchStatus::Error },
        }),
        // enum 'SearchMethod'
        QtMocHelpers::EnumData<SearchMethod>(11, 11, QMC::EnumFlags{}).add({
            {   12, SearchMethod::Indexed },
            {   13, SearchMethod::Realtime },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN9dfmsearchE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN9dfmsearchE =
    dfmsearch::qt_create_metaobjectdata<qt_meta_tag_ZN9dfmsearchE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN9dfmsearchE =
    qt_staticMetaObjectContent_ZN9dfmsearchE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN9dfmsearchE =
    qt_staticMetaObjectContent_ZN9dfmsearchE.relocatingData;

Q_CONSTINIT const QMetaObject dfmsearch::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN9dfmsearchE.stringdata,
    qt_staticMetaObjectStaticContent_ZN9dfmsearchE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN9dfmsearchE.metaTypes,
    nullptr
} };

QT_WARNING_POP
