/****************************************************************************
** Meta object code from reading C++ file 'searchengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../util-dfm/include/dfm-search/dfm-search/searchengine.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9dfmsearch12SearchEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto dfmsearch::SearchEngine::qt_create_metaobjectdata<qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dfmsearch::SearchEngine",
        "searchStarted",
        "",
        "resultsFound",
        "dfmsearch::SearchResultList",
        "results",
        "statusChanged",
        "SearchStatus",
        "status",
        "searchFinished",
        "searchCancelled",
        "errorOccurred",
        "dfmsearch::SearchError",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'searchStarted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resultsFound'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchResultList &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(SearchStatus)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchResultList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'searchCancelled'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const dfmsearch::SearchError &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SearchEngine, qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dfmsearch::SearchEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>.metaTypes,
    nullptr
} };

void dfmsearch::SearchEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchStarted(); break;
        case 1: _t->resultsFound((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchResultList>>(_a[1]))); break;
        case 2: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<SearchStatus>>(_a[1]))); break;
        case 3: _t->searchFinished((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchResultList>>(_a[1]))); break;
        case 4: _t->searchCancelled(); break;
        case 5: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<dfmsearch::SearchError>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< dfmsearch::SearchError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)()>(_a, &SearchEngine::searchStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)(const dfmsearch::SearchResultList & )>(_a, &SearchEngine::resultsFound, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)(SearchStatus )>(_a, &SearchEngine::statusChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)(const dfmsearch::SearchResultList & )>(_a, &SearchEngine::searchFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)()>(_a, &SearchEngine::searchCancelled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchEngine::*)(const dfmsearch::SearchError & )>(_a, &SearchEngine::errorOccurred, 5))
            return;
    }
}

const QMetaObject *dfmsearch::SearchEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmsearch::SearchEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9dfmsearch12SearchEngineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmsearch::SearchEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmsearch::SearchEngine::searchStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmsearch::SearchEngine::resultsFound(const dfmsearch::SearchResultList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void dfmsearch::SearchEngine::statusChanged(SearchStatus _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dfmsearch::SearchEngine::searchFinished(const dfmsearch::SearchResultList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void dfmsearch::SearchEngine::searchCancelled()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dfmsearch::SearchEngine::errorOccurred(const dfmsearch::SearchError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
