// This file is autogenerated. Changes will be overwritten.
#include "FDDTZOXWLG/moc_denumeratorfuture.cpp"
#include "FDDTZOXWLG/moc_dfilefuture.cpp"
#include "FDDTZOXWLG/moc_dwatcher.cpp"
#include "PGVKHOT7RA/moc_denumerator_p.cpp"
